# -*- coding: utf-8 -*-
from .asset import Asset


class Shot(Asset):
    """
    This class describes a Shot object child of Item class.

    .. note::
        This class is using all the same methods than the :class:`~aquarium.items.asset.Asset` class
    """
    pass
